#import <objc/Object.h>
#import <objc/hashtable.h>
#import "graphics.h"

@class NXImage, View;

@protocol NXWorkspaceRequestProtocol

- (BOOL)openFile:(const char *)fullPath;
- (BOOL)openFile:(const char *)fullPath withApplication:(const char *)appName;
- (BOOL)openFile:(const char *)fullPath withApplication:(const char *)appName andDeactivate:(BOOL)flag;

- (BOOL)openTempFile:(const char *)fullPath;

- (BOOL)openFile:(const char *)fullPath
       fromImage:(NXImage *)anImage
	      at:(const NXPoint *)point
	  inView:(View *)aView;

- (BOOL)launchApplication:(const char *)appName;
- (BOOL)launchApplication:(const char *)appName showTile:(BOOL)showTile autolaunch:(BOOL)autolaunch;
- (const char *)getFullPathForApplication:(const char *)appName;

- (BOOL)findString:(const char *)aString inFile:(const char *)fullPath;
- (BOOL)selectFile:(const char *)fullPath inFileViewerRootedAt:(const char *)rootFullpath;

- (void)findApplications;

- (void)fileSystemChanged;
- (BOOL)didFileSystemChange;
- (void)defaultsChanged;
- (BOOL)didDefaultsChange;

/* Return values for type in getInfoForFile: */

extern NXAtom NXPlainFileType, NXDirectoryFileType, NXApplicationFileType;
extern NXAtom NXFilesystemFileType, NXShellCommandFileType;

- (BOOL)getInfoForFile:(const char *)fullPath application:(char **)appName type:(NXAtom *)type;

- (NXImage *)getIconForFile:(const char *)fullPath;

- (BOOL)getInfoForFileSystemAt:(const char *)fullPath
		   isRemovable:(BOOL *)removableFlag
		    isWritable:(BOOL *)writableFlag
		 isUnmountable:(BOOL *)unmountableFlag
		   description:(char **)description
			  type:(char **)fileSystemType;

#define WSM_MOVE_OPERATION "move"
#define WSM_COPY_OPERATION "copy"
#define WSM_LINK_OPERATION "link"
#define WSM_COMPRESS_OPERATION "compress"
#define WSM_DECOMPRESS_OPERATION "decompress"
#define WSM_ENCRYPT_OPERATION "encrypt"
#define WSM_DECRYPT_OPERATION "decrypt"
#define WSM_DESTROY_OPERATION "destroy"
#define WSM_RECYCLE_OPERATION "recycle"
#define WSM_DUPLICATE_OPERATION "duplicate"

- (int)performFileOperation:(const char *)operation
		     source:(const char *)source
		destination:(const char *)destination
		      files:(const char *)files
		    options:(const char *)options;

- (BOOL)unmountAndEjectDeviceAt:(const char *)path;
- (int)extendPowerOffBy:(int)requested;

- (void)slideImage:(NXImage *)image from:(const NXPoint *)fromPoint to:(const NXPoint *)toPoint;

- (void)hideOtherApplications;

- (void)beginListeningForDeviceStatusChanges;
- (void)endListeningForDeviceStatusChanges;

- (void)beginListeningForApplicationStatusChanges;
- (void)endListeningForApplicationStatusChanges;

- (BOOL)getMountedRemovableMedia:(char **)mountedList;
- (BOOL)mountNewRemovableMedia:(char **)newlyMountedList;
- (void)checkForRemovableMedia;

@end
