/*	Copyright (c) 1991 NeXT Computer, Inc.  All rights reserved.   
 *
 * i82077Regs.h - Intel 82077 register defintions, i386 version.
 *
 * HISTORY
 * 3-Sept-92    David Somayajulu at NeXT
 *      Created. 
 */

#ifndef _BSD_DEV_I386_I82077_REGS_
#define	_BSD_DEV_I386_I82077_REGS_

/* Port Addresses for the Floppy Controller 82077A for x86 version */

#define FC_DIGITAL_OUTPUT_REG		0x03F2 	/*Write Only */
			/* Floppy Controller Digital Output Register */
#define FC_CONTROL_STATUS_REG		0x03F4	/*Read Only */
			/* Floppy Controller Status Register */
#define FC_DATA_REG			0x03F5	/* Read/Write */
			/* Floppy Controller Data Register */
#define FC_DIGITAL_INPUT_REG		0x03F7	/* Read Only */
			/* Floppy Controller Digital Input Register */
#define FC_DATA_RATE_SELECT_REG		0x03F7	/* Write Only */
			/* Floppy Controller Data Rate Select Register */


/*
 * Digital Output Register (Write Only)
 */
#define DOR_RESERVERD0		0x80	// = 0 Reserved
#define DOR_RESERVERD1		0x40 	// = 0 Reserved
#define DOR_MOTEN1		0x20	// == 1 Drive 1 motor enable  
#define DOR_MOTEN0		0x10	// == 1 Drive 0 motor enable  
#define DOR_MOTEN_MASK		(DOR_MOTEN0 | DOR_MOTEN1)
#define DOR_DMAENABLE		0x08	// == 1 Diskette DMA Enable 
#define DOR_CNTRLR_RST_NOT	0x04	// == 0 Controller Reset 
#define DOR_RESERVED2		0x02	// = 0 Reserved 
#define DOR_DRVSEL_MASK		0x01	// Drive Select
#define DOR_SELDRV0		0x00	// == 0 Select Drive 0
#define DOR_SELDRV1		0x01	// == 1 Select Drive 1

/*
 * Status Register (Read Only)
 */
#define SR_DATA_REG_READY	0x80	// ==1 Data Register is Ready 
#define SR_TRANSFER_DIR		0x40	// direction of data transfer 
#define SR_TOCNTRLR		0x00	// ==0 from system to controller
#define SR_FROMCNTRLR		0x40	// ==1 from controller to system
#define SR_NONDMA		0x20	// ==1 Non-DMA mode
#define SR_FCBUSY		0x10	// ==1 Floppy Controller Busy
#define SR_RESERVED0		0x08	// =0 Reverved
#define SR_RESERVED1		0x04	// =0 Reverved
#define SR_DRIVE1_BUSY		0x02	// ==1 Drive 1 Busy 
#define SR_DRIVE0_BUSY		0x02	// ==1 Drive 0 Busy 

/*
 * Digital Input Register (Read only)
*/
#define DIR_FLOPPY_CHANGE	0x80	// ==1 The floppy has been changed
/* The remaining bits in this register are reserved */

/*
 * Data Rate Select Register (Write Only)
 */

#define DSR_DRATE_SELECT	0x03	// data rate select see below 

/* possible data rates */
#define 	DRATE_MFM_500	0x00	// 500 Kb/sec MFM
#define 	DRATE_MFM_300	0x01	// 300 Kb/sec MFM
#define 	DRATE_MFM_250	0x02	// 250 Kb/sec MFM
#define 	DRATE_MFM_1000	0x03	// 1000 Kb/sec MFM

#define 	DR_FM_250KBS	0x00	// 250 Kb/sec FM
#define 	DR_FM_150KBS	0x01	// 150 Kb/sec FM
#define 	DR_FM_125KBS	0x02	// 125 Kb/sec FM

/* The remaining bits in this register are reserved */

#define 	FLOPPY_BUFSIZE 	64 /*??? needs to be moved elsewhere */
#define 	FC_RESET_HOLD	500 /* reset should be held true for this long */


/* this structure is used to keep track of the values written to registers 
   in 82077 which are write only */
typedef struct _atfloppyregs {
	unsigned char dor; /* digital output register */
	unsigned char dsr; /* data rate select register */
} ATFloppyRegs;


#define WRITE_FREG(reg, val)	{\
				outb(reg,val);\
				}

#define READ_FREG(reg,val)	{\
				val = inb(reg);\
				}
				
#define READID_TIMEOUT		1000

#endif	_BSD_DEV_I386_I82077_REGS_
