#
#                                 bundle.make
#
# Inputs from Makefile.preamble (no defaults):
#	BUNDLELDFLAGS - Additional flags passed to ld when linking bundles
#	OTHER_CFLAGS - Flags passed to compiler (in addition to -g, -O, etc.)
#	OTHER_LDFLAGS - Flags passed to ld (in addition to -ObjC, etc.)
#	OTHER_OFILES - Additional relocatables that may be linked in
#	OTHER_PRODUCT_DEPENDS - Other dependencies of this project
#	OTHER_SOURCEFILES - Other source files maintained by .pre/postamble
#	OTHER_GARBAGE - Additional files to be removed by make 'clean' 
#
#	MSGFILES - .msg files that should have msgwrap run on them
#	DEFSFILES - .defs files that should have mig run on them
#	MIGFILES - .mig files (no .defs files) that should have mig run on them
#
#	PRECOMPS - Precompiled headers that should be built before compilation 
#	OTHER_INITIAL_TARGETS - Targets to be built before subprojects, etc.
#
#	DSTROOT - Virtual root directory prepended to $(INSTALLDIR)
#
# Inputs from Makefile.postamble (with defaults set by this file):
#	MAKEFILES - Makefiles this project depends on (default = Makefile)
#	DEFAULT_CFLAGS - All compiler flags for this level (default set below)
#
# Inputs from Makefile (avoid setting these, ProjectBuilder controls them):
#	NAME - name of application
#	LANGUAGE - langage in which the project is written (default "English")
#	LOCAL_RESOURCES - localized resources (e.g. nib's, images) of project
#	GLOBAL_RESOURCES - non-localized resources of project
#
#	CLASSES - Class implementation files in project.
#	HFILES - Header files in project.
#	MFILES - Other Objective-C source files in project. 
#	CFILES - Other C source files in project. 
#	PSWFILES - .psw files in the project
#	PSWMFILES - .pswm files in the project
#	SUBPROJECTS - Subprojects of this project
#	BUNDLES - Bundle subprojects of this project
#	OTHERSRCS - Other miscellaneous sources of this project
#	OTHERLINKED - Source files not matching a standard source extention
#	OTHERLINKEDOFILES - Other relocatable files to (always) link in.
#
#	APP_MAKEFILE_DIR - Directory in which to find generic set of Makefiles
#	MAKEFILEDIR - Directory in which to find $(MAKEFILE)
#	MAKEFILE - Top level mechanism Makefile (e.g., app.make, bundle.make)
#	INSTALLDIR - Directory app will be installed into by 'install' target
#
# Outputs to common.make:
#	PRODUCT_ROOT 
#


# Defaults for NeXT-specific Release Control options
SRCROOT = .
SYMROOT = .
OBJROOT = .

# Set these in case this is a standalone case (much of this cruftiness has to do with sh's inability to operate on $$bun in the common.make subcomponents rule)

BUNDLE_DIR = $(SYMROOT)/$(NAME).bproj
PRODUCT_ROOT = $(BUNDLE_DIR:.bproj=.bundle)
BUNDLE = "$(PRODUCT_ROOT)/$(NAME:.bproj=)"

# ...and versions for installing the bundle.
REL_BUNDLE_DIR = $(NAME).bproj
REL_PRODUCT_ROOT = $(REL_BUNDLE_DIR:.bproj=.bundle)
REL_BUNDLE = $(REL_PRODUCT_ROOT)/$(NAME:.bproj=)


APP_MAKEFILE_DIR = /NextDeveloper/Makefiles/app
include $(APP_MAKEFILE_DIR)/common.make
include $(APP_MAKEFILE_DIR)/specialrules.make

all:: bundle

# bundle targets:

bundle:: 
	@(evaled_rc_archs="$(RC_ARCHS)"; \
	obj_dir=`echo $$evaled_rc_archs | sed 's/ /_/g'`_obj ; \
	$(MAKE) project \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(NORMAL_CFLAGS)" \
		"MAKEFILEDIR = $(MAKEFILEDIR)" \
		"RC_CFLAGS = $(RC_CFLAGS)" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"PRODUCT_ROOT = $(PRODUCT_ROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

debug::
	@(evaled_rc_archs="$(RC_ARCHS)"; \
	obj_dir=`echo $$evaled_rc_archs | sed 's/ /_/g'`_debug_obj ; \
	$(MAKE) project \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(DEBUG_CFLAGS)" \
		"MAKEFILEDIR = $(MAKEFILEDIR)" \
		"RC_CFLAGS = $(RC_CFLAGS)" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"PRODUCT_ROOT = $(PRODUCT_ROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )
		
project:: $(INITIAL_TARGETS) subprojs bundles resources $(BUNDLE)

$(BUNDLE): $(PRODUCT_DEPENDS) $(PRODUCT_ROOT) $(SYM_DIR) 
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; then \
	   cmd="$(CC) $(ALL_CFLAGS) -nostdlib $(BUNDLELDFLAGS) $(OFILES) $(OTHER_OFILES) -r -o $(BUNDLE)" ; \
	   echo $$cmd ; $$cmd ; \
	else \
	   $(ECHO) Warning: Building empty bundle. ; \
	   $(TOUCH) $(SYM_DIR)/$(NAME:.bproj=).c ; \
	   $(CC) $(ALL_CFLAGS) -c $(SYM_DIR)/$(NAME:.bproj=).c -o $(BUNDLE) ; \
	fi ; \
	$(CHMOD) a+x $(BUNDLE) )

depend:: Makefile.dependencies $(SUBPROJECTS:.subproj=.depend) $(BUNDLES:.bproj=.depend)

$(PRODUCT_ROOT):
	$(MKDIRS) $(PRODUCT_ROOT)

$(DSTROOT)$(INSTALLDIR):
	$(MKDIRS) $(DSTROOT)$(INSTALLDIR)

before_install:: $(DSTROOT)$(INSTALLDIR)
	$(RM) -rf $(DSTROOT)$(INSTALLDIR)/$(REL_PRODUCT_ROOT)

after_install::

# Note that the following will not correctly install a standalone
# bundle under a Jasper build.  

install:: bundle before_install installhdrs $(OTHER_INSTALL_DEPENDS)
	(cd $(SYMROOT); $(TAR) chf - $(REL_PRODUCT_ROOT)) | (cd $(DSTROOT)$(INSTALLDIR); $(TAR) xf -)
	@somebundles=no; \
	for i in $(BUNDLES:.bproj=) none ; do \
		if [ $$i = none ] ; then continue; fi; \
		$(ECHO) $(STRIP) $(RELOCATABLE_STRIP_OPTS) $(DSTROOT)$(INSTALLDIR)/$(REL_PRODUCT_ROOT)/$$i.bundle/$$i; \
		$(STRIP) $(RELOCATABLE_STRIP_OPTS) $(DSTROOT)$(INSTALLDIR)/$(REL_PRODUCT_ROOT)/$$i.bundle/$$i; \
	done; 
	$(STRIP) $(RELOCATABLE_STRIP_OPTS) \
	        $(DSTROOT)$(INSTALLDIR)/$(REL_BUNDLE)
	-$(CHOWN) -Rf $(INSTALL_AS_USER).$(INSTALL_AS_GROUP) \
		$(DSTROOT)$(INSTALLDIR)/$(REL_PRODUCT_ROOT)
	-$(CHMOD) -Rf go-w $(DSTROOT)$(INSTALLDIR)/$(REL_BUNDLE)
	@(if [ "$(INSTALL_PERMISSIONS)" != "" ] ; then \
		$(ECHO) $(CHMOD) -f $(INSTALL_PERMISSIONS) \
		  $(DSTROOT)$(INSTALLDIR)/$(REL_BUNDLE); \
		$(CHMOD) -f $(INSTALL_PERMISSIONS) \
		  $(DSTROOT)$(INSTALLDIR)/$(REL_BUNDLE); \
	fi)
	@(evaled_rc_archs="$(RC_ARCHS)"; \
	obj_dir=`echo $$evaled_rc_archs | sed 's/ /_/g'`_obj ; \
	$(MAKE) after_install 	"DSTROOT=$(DSTROOT)" \
			       	"OBJROOT=$(OBJROOT)" \
			       	"SYMROOT=$(SYMROOT)" \
				"PRODUCT_ROOT = $(PRODUCT_ROOT)" \
				"SYM_DIR = $(SYMROOT)/sym" \
				"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
				"PROJ_CFLAGS = $(NORMAL_CFLAGS)" \
				"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
				"RC_CFLAGS = $(RC_CFLAGS)" \
				"RC_ARCHS = $(RC_ARCHS)" )


installhdrs::

#.DEFAULT 

help:
	@if (test "$<"); then		\
	    $(ECHO) 'ERROR: Unrecognized Makefile target "$<"';	\
	fi
	@$(ECHO) 'The following targets are available:'
	@$(ECHO) 
	@$(ECHO) ' all         Makes the bundle target (below).  This is the default target.'
	@$(ECHO) 
	@$(ECHO) ' bundle      Compiles and links an optimized (and debuggable) version of the'
	@$(ECHO) '             bundle into $(NAME).bundle'
	@$(ECHO) 	
	@$(ECHO) ' depend      Generates an optional Makefile.dependencies file for every level'
	@$(ECHO) '             of the bundle to reflect the complete dependency graph'
	@$(ECHO) '             (including headers).'
	@$(ECHO) 
	@$(ECHO) ' install     Builds (if needed) and copies <BundleName>.bundle'
	@$(ECHO) '             into the installation directory, setting permissions'
	@$(ECHO) '             and owners as appropriate.'
	@$(ECHO) 	
	@$(ECHO) ' installsrc  Installs the source files for the bundle into the directory'
	@$(ECHO) '             specified in the SRCROOT variable in a command-line argument.'
	@$(ECHO) '             Note: the target directory will be removed and recreated.'
	@$(ECHO) 
	@$(ECHO) ' installhdrs Installs the header files for the bundle.'
	@$(ECHO) 
	@$(ECHO) ' clean       Removes all files in the bundle derived from previous builds.'
	@$(ECHO) 
	@$(ECHO) ' help        Lists these targets with their descriptions.'
	@$(ECHO) 
	@$(ECHO) 'For more details, see the comments in /NextDeveloper/Makefiles/app/*.make.'
	